@@ -74,7 +74,7 @@ from cylc.flow.network import API
 from cylc.flow.network.authentication import key_housekeeping
 from cylc.flow.network.schema import WorkflowStopMode
 from cylc.flow.network.server import WorkflowRuntimeServer
-from cylc.flow.option_parsers import verbosity_to_env
+from cylc.flow.option_parsers import verbosity_to_env, verbosity_to_opts
 from cylc.flow.parsec.exceptions import ParsecError
 from cylc.flow.parsec.OrderedDict import DictTree
 from cylc.flow.parsec.validate import DurationFloat
@@ -409,28 +409,29 @@ class Scheduler:
                 self.config.cfg['scheduler']['cycle point time zone'])
 
         # Note that the following lines must be present at the top of
-        # the workflow log file for use in reference test runs:
+        # the workflow log file for use in reference test runs.
+        # (These are also headers that get logged on each rollover)
         LOG.info(
             f'Run mode: {self.config.run_mode()}',
-            extra=RotatingLogFileHandler.extra
+            extra=RotatingLogFileHandler.header_extra
         )
         LOG.info(
             f'Initial point: {self.config.initial_point}',
-            extra=RotatingLogFileHandler.extra
+            extra=RotatingLogFileHandler.header_extra
         )
         if self.config.start_point != self.config.initial_point:
             LOG.info(
                 f'Start point: {self.config.start_point}',
-                extra=RotatingLogFileHandler.extra
+                extra=RotatingLogFileHandler.header_extra
             )
         LOG.info(
             f'Final point: {self.config.final_point}',
-            extra=RotatingLogFileHandler.extra
+            extra=RotatingLogFileHandler.header_extra
         )
         if self.config.stop_point:
             LOG.info(
                 f'Stop point: {self.config.stop_point}',
-                extra=RotatingLogFileHandler.extra
+                extra=RotatingLogFileHandler.header_extra
             )
 
         self.broadcast_mgr.linearized_ancestors.update(
@@ -543,32 +544,38 @@ class Scheduler:
             LOG.setLevel(logging.INFO)
 
         # Print workflow name to disambiguate in case of inferred run number
+        # while in no-detach mode
         LOG.info(f"Workflow: {self.workflow}")
+        # Headers that also get logged on each rollover:
         LOG.info(
             self.START_MESSAGE_TMPL % {
                 'comms_method': 'tcp',
                 'host': self.host,
                 'port': self.server.port,
                 'pid': os.getpid()},
-            extra=RotatingLogFileHandler.extra,
+            extra=RotatingLogFileHandler.header_extra,
         )
         LOG.info(
             self.START_PUB_MESSAGE_TMPL % {
                 'comms_method': 'tcp',
                 'host': self.host,
                 'port': self.server.pub_port},
-            extra=RotatingLogFileHandler.extra,
+            extra=RotatingLogFileHandler.header_extra,
         )
         restart_num = self.get_restart_num() + 1
         LOG.info(
-            'Run: (re)start number=%d, log rollover=%d',
-            restart_num,
-            1,  # hard code 1 which is updated later if required
-            extra=RotatingLogFileHandler.extra_num
+            f'Run: (re)start number={restart_num}, log rollover=%d',
+            # Hard code 1 in args, gets updated on log rollover (NOTE: this
+            # must be the only positional arg):
+            1,
+            extra={
+                **RotatingLogFileHandler.header_extra,
+                RotatingLogFileHandler.ROLLOVER_NUM: 1
+            }
         )
         LOG.info(
             f'Cylc version: {CYLC_VERSION}',
-            extra=RotatingLogFileHandler.extra
+            extra=RotatingLogFileHandler.header_extra
         )
 
         if is_quiet:
@@ -599,16 +606,22 @@ class Scheduler:
         except SchedulerStop as exc:
             # deliberate stop
             await self.shutdown(exc)
-            if self.auto_restart_mode == AutoRestartMode.RESTART_NORMAL:
-                self.workflow_auto_restart()
-            # run shutdown coros
-            await asyncio.gather(
-                *main_loop.get_runners(
-                    self.main_loop_plugins,
-                    main_loop.CoroTypes.ShutDown,
-                    self
+            try:
+                if self.auto_restart_mode == AutoRestartMode.RESTART_NORMAL:
+                    self.workflow_auto_restart()
+                # run shutdown coros
+                await asyncio.gather(
+                    *main_loop.get_runners(
+                        self.main_loop_plugins,
+                        main_loop.CoroTypes.ShutDown,
+                        self
+                    )
                 )
-            )
+            except Exception as exc:
+                # Need to log traceback manually because otherwise this
+                # exception gets swallowed
+                LOG.exception(exc)
+                raise
 
         except (KeyboardInterrupt, asyncio.CancelledError, Exception) as exc:
             # Includes SchedulerError
@@ -1108,10 +1121,7 @@ class Scheduler:
         config_dir = get_workflow_run_config_log_dir(
             self.workflow)
         config_logs = get_sorted_logs_by_time(config_dir, "*[0-9].cylc")
-        if config_logs:
-            log_num = get_next_log_number(config_logs[-1])
-        else:
-            log_num = '01'
+        log_num = get_next_log_number(config_logs[-1]) if config_logs else 1
         if is_reload:
             load_type = "reload"
             load_type_num = get_reload_start_number(config_logs)
@@ -1123,7 +1133,7 @@ class Scheduler:
             load_type = "start"
             load_type_num = '01'
         file_name = get_workflow_run_config_log_dir(
-            self.workflow, f"{log_num}-{load_type}-{load_type_num}.cylc")
+            self.workflow, f"{log_num:02d}-{load_type}-{load_type_num}.cylc")
         with open(file_name, "w") as handle:
             handle.write("# cylc-version: %s\n" % CYLC_VERSION)
             self.config.pcfg.idump(sparse=True, handle=handle)
@@ -1232,7 +1242,7 @@ class Scheduler:
                 return
         self.workflow_event_handler.handle(self, event, str(reason))
 
-    def release_queued_tasks(self):
+    def release_queued_tasks(self) -> None:
         """Release queued tasks, and submit jobs.
 
         The task queue manages references to task proxies in the task pool.
@@ -1257,36 +1267,48 @@ class Scheduler:
             and self.auto_restart_time is None
         ):
             pre_prep_tasks = self.pool.release_queued_tasks()
-            if not pre_prep_tasks:
-                # No tasks to submit.
-                return
 
-            # Start the job submission process.
-            self.is_updated = True
-            self.reset_inactivity_timer()
+        elif (
+            self.should_auto_restart_now()
+            and self.auto_restart_mode == AutoRestartMode.RESTART_NORMAL
+        ):
+            # Need to get preparing tasks to submit before auto restart
+            pre_prep_tasks = [
+                itask for itask in self.pool.get_tasks()
+                if itask.state(TASK_STATUS_PREPARING)
+            ]
 
-            self.task_job_mgr.task_remote_mgr.rsync_includes = (
-                self.config.get_validated_rsync_includes())
+        # Return, if no tasks to submit.
+        else:
+            return
+        if not pre_prep_tasks:
+            return
 
-            meth = LOG.debug
-            if self.options.reftest or self.options.genref:
-                meth = LOG.info
-            for itask in self.task_job_mgr.submit_task_jobs(
-                self.workflow,
-                pre_prep_tasks,
-                self.server.curve_auth,
-                self.server.client_pub_key_dir,
-                self.config.run_mode('simulation')
-            ):
-                if itask.flow_nums:
-                    flow = ','.join(str(i) for i in itask.flow_nums)
-                else:
-                    flow = FLOW_NONE
-                meth(
-                    f"{itask.identity} -triggered off "
-                    f"{itask.state.get_resolved_dependencies()}"
-                    f" in flow {flow}"
-                )
+        # Start the job submission process.
+        self.is_updated = True
+        self.reset_inactivity_timer()
+
+        self.task_job_mgr.task_remote_mgr.rsync_includes = (
+            self.config.get_validated_rsync_includes())
+
+        log = LOG.debug
+        if self.options.reftest or self.options.genref:
+            log = LOG.info
+        for itask in self.task_job_mgr.submit_task_jobs(
+            self.workflow,
+            pre_prep_tasks,
+            self.server.curve_auth,
+            self.server.client_pub_key_dir,
+            is_simulation=self.config.run_mode('simulation')
+        ):
+            if itask.flow_nums:
+                flow = ','.join(str(i) for i in itask.flow_nums)
+            else:
+                flow = FLOW_NONE
+            log(
+                f"{itask.identity} -triggered off "
+                f"{itask.state.get_resolved_dependencies()} in flow {flow}"
+            )
 
     def process_workflow_db_queue(self):
         """Update workflow DB."""
@@ -1373,27 +1395,33 @@ class Scheduler:
             self.time_next_kill = time() + self.INTERVAL_STOP_KILL
 
         # Is the workflow set to auto stop [+restart] now ...
-        if self.auto_restart_time is None or time() < self.auto_restart_time:
+        if not self.should_auto_restart_now():
             # ... no
             pass
         elif self.auto_restart_mode == AutoRestartMode.RESTART_NORMAL:
-            # ... yes - wait for local jobs to complete before restarting
-            #           * Avoid polling issues see #2843
-            #           * Ensure the host can be safely taken down once the
-            #             workflow has stopped running.
+            # ... yes - wait for preparing jobs to see if they're local and
+            # wait for local jobs to complete before restarting
+            #    * Avoid polling issues - see #2843
+            #    * Ensure the host can be safely taken down once the
+            #      workflow has stopped running.
             for itask in self.pool.get_tasks():
+                if itask.state(TASK_STATUS_PREPARING):
+                    LOG.info(
+                        "Waiting for preparing jobs to submit before "
+                        "attempting restart"
+                    )
+                    break
                 if (
-                        itask.state(*TASK_STATUSES_ACTIVE)
-                        and itask.summary['job_runner_name']
-                        and not is_remote_platform(itask.platform)
-                        and self.task_job_mgr.job_runner_mgr
-                        .is_job_local_to_host(
-                            itask.summary['job_runner_name'])
+                    itask.state(*TASK_STATUSES_ACTIVE)
+                    and itask.summary['job_runner_name']
+                    and not is_remote_platform(itask.platform)
+                    and self.task_job_mgr.job_runner_mgr.is_job_local_to_host(
+                        itask.summary['job_runner_name'])
                 ):
                     LOG.info('Waiting for jobs running on localhost to '
                              'complete before attempting restart')
                     break
-            else:
+            else:  # no break
                 self._set_stop(StopMode.REQUEST_NOW_NOW)
         elif (  # noqa: SIM106
             self.auto_restart_mode == AutoRestartMode.FORCE_STOP
@@ -1405,9 +1433,19 @@ class Scheduler:
             raise SchedulerError(
                 'Invalid auto_restart_mode=%s' % self.auto_restart_mode)
 
-    def workflow_auto_restart(self, max_retries=3):
+    def should_auto_restart_now(self) -> bool:
+        """Is it time for the scheduler to auto stop + restart?"""
+        return (
+            self.auto_restart_time is not None and
+            time() >= self.auto_restart_time
+        )
+
+    def workflow_auto_restart(self, max_retries: int = 3) -> bool:
         """Attempt to restart the workflow assuming it has already stopped."""
-        cmd = ['cylc', 'play', quote(self.workflow)]
+        cmd = [
+            'cylc', 'play', quote(self.workflow),
+            *verbosity_to_opts(cylc.flow.flags.verbosity)
+        ]
         if self.options.abort_if_any_task_fails:
             cmd.append('--abort-if-any-task-fails')
         for attempt_no in range(max_retries):
@@ -1419,6 +1457,7 @@ class Scheduler:
                 stdin=DEVNULL,
                 stdout=PIPE,
                 stderr=PIPE,
+                text=True
             )
             # * new_host comes from internal interface which can only return
             #   host names
@@ -1429,7 +1468,8 @@ class Scheduler:
                         f' will retry in {self.INTERVAL_AUTO_RESTART_ERROR}s')
                 LOG.critical(
                     f"{msg}. Restart error:\n",
-                    f"{proc.communicate()[1].decode()}")
+                    f"{proc.communicate()[1]}"
+                )
                 sleep(self.INTERVAL_AUTO_RESTART_ERROR)
             else:
                 LOG.info(f'Workflow now running on "{new_host}".')
@@ -1681,28 +1721,26 @@ class Scheduler:
 
     async def _shutdown(self, reason: Exception) -> None:
         """Shutdown the workflow."""
+        shutdown_msg = "Workflow shutting down"
         if isinstance(reason, SchedulerStop):
-            LOG.info(f'Workflow shutting down - {reason.args[0]}')
+            LOG.info(f'{shutdown_msg} - {reason.args[0]}')
             # Unset the "paused" status of the workflow if not auto-restarting
             if self.auto_restart_mode != AutoRestartMode.RESTART_NORMAL:
                 self.resume_workflow(quiet=True)
         elif isinstance(reason, SchedulerError):
-            LOG.error(f"Workflow shutting down - {reason}")
+            LOG.error(f"{shutdown_msg} - {reason}")
         elif isinstance(reason, CylcError) or (
             isinstance(reason, ParsecError) and reason.schd_expected
         ):
-            LOG.error(
-                f"Workflow shutting down - {type(reason).__name__}: {reason}"
-            )
+            LOG.error(f"{shutdown_msg} - {type(reason).__name__}: {reason}")
             if cylc.flow.flags.verbosity > 1:
                 # Print traceback
                 LOG.exception(reason)
         else:
             LOG.exception(reason)
             if str(reason):
-                LOG.critical(f'Workflow shutting down - {reason}')
-            else:
-                LOG.critical('Workflow shutting down')
+                shutdown_msg += f" - {reason}"
+            LOG.critical(shutdown_msg)
 
         if hasattr(self, 'proc_pool'):
             self.proc_pool.close()
